@@ -50,6 +50,10 @@ VALID_HOSTS = (
     "viewer.heros-web.com",
 )
 
+_MAGAZINE_TITLE_PATTERN = (
+    r"([0-9０-９]+年)?([0-9０-９]+(・[0-9０-９]+合併)?月?号|(Ｎｏ|ｖｏｌ)．[0-9０-９]+)$"
+)
+
 
 class _Page(TypedDict):
     height: int
@@ -96,13 +100,14 @@ class GetJump:
         nxt = j["nextReadableProductUri"]
         nxt = self.__check_next(nxt)
         if j["typeName"] == "magazine":
-            series_title = re.sub(
-                r"\s*([0-9０-９]+年)?([0-9０-９]+月?号|(Ｎｏ|ｖｏｌ)．[0-9０-９]+)$", "", j["title"]
-            )
-        elif j["typeName"] == "episode":
+            series_title = re.sub(r"\s*" + _MAGAZINE_TITLE_PATTERN, "", j["title"])
+            title = j["title"].replace(series_title, "", 1).strip()
+        elif j["typeName"] == "episode" or "volume":
             series_title = j["series"]["title"].replace("/", "／")
-        title = j["title"].replace("/", "／")
+            title = j["title"].replace("/", "／")
         # print(f"[series={repr(series_title)}, title={repr(title)}]")
+        series_title = self.__normalize_fname(series_title)
+        title = self.__normalize_fname(title)
 
         save_dir = os.path.join(save_path, series_title, title)
         if os.path.exists(save_dir) and not overwrite:
@@ -256,3 +261,14 @@ class GetJump:
             for x, cropped in enumerate(inbuff):
                 img.paste(cropped, box=(fixed_width * x, fixed_height * y))
         return img
+
+    def __normalize_fname(self, fname: str) -> str:
+        if fname == "":
+            raise ValueError(f"{repr(fname)} is empty.")
+        elif fname.endswith("."):
+            return self.__normalize_fname(fname[:-1])
+        elif fname.startswith(" "):
+            return self.__normalize_fname(fname[1:])
+        elif fname.endswith(" "):
+            return self.__normalize_fname(fname[:-1])
+        return fname
